const St      = imports.gi.St;
const Meta    = imports.gi.Meta;
const Pango   = imports.gi.Pango;
const Clutter = imports.gi.Clutter;
const Lang    = imports.lang;


// =====================================================================
// Multi-line Entry
//
// @hint_text        : string
// @scrollable       : bool (make entry use scrollbar or grow indefinitely)
// @single_line_mode : bool (removes any line breaks, still wraps)
//
// If the entry is set to multiline, you can still connect on the 'activate'
// signal of the StEntry in case you need it for completion, but then you must
// remove the newly inserted newline character yourself.
// =====================================================================
var MultiLineEntry = new Lang.Class({
    Name: 'Timepp.MultiLineEntry',

    _init: function(hint_text, scrollable, single_line_mode) {
        this.scrollable       = scrollable;
        this.single_line_mode = single_line_mode;

        this.sanitize_flag            = false;
        this.automatic_newline_insert = true;
        this.keep_min_height          = true;


        //
        // draw
        //
        this.actor = new St.BoxLayout({ y_expand: true, vertical: true });

        this.entry_container = new St.BoxLayout({ reactive: true, vertical: true });

        if (scrollable) {
            this.scroll_box = new St.ScrollView({ x_fill: true, y_align: St.Align.START, style_class: 'multiline-entry-scrollbox vfade'});
            this.actor.add(this.scroll_box);
            this.scroll_box.add_actor(this.entry_container);
        } else {
            this.actor.add_actor(this.entry_container);
        }

        this.entry = new St.Entry({ can_focus: true, hint_text: hint_text, name: 'menu-search-entry' });
        this.entry_container.add_actor(this.entry);

        this.text_constraint = new Clutter.BindConstraint({ source: this.entry, coordinate: Clutter.BindCoordinate.POSITION });
        this.entry.clutter_text.add_constraint(this.text_constraint);

        this.entry.clutter_text.activatable      = true;
        this.entry.clutter_text.single_line_mode = false;
        this.entry.clutter_text.line_wrap        = true;
        this.entry.clutter_text.line_wrap_mode   = Pango.WrapMode.WORD_CHAR;


        //
        // listen
        //
        if (single_line_mode) {
            this.entry.clutter_text.connect('insert-text', (_, ...args) => this._before_text_changed(...args));
        }
        this.entry.clutter_text.connect('activate', () => { if (this.automatic_newline_insert) this.insert_newline(); });
        this.entry.clutter_text.connect('text-changed', () => this._after_text_changed());
        this.entry_container.connect('button-press-event', () => this.entry.grab_key_focus());
        this.entry.connect('allocation_changed', () => {
            this.text_constraint.offset = this.entry.get_theme_node().get_vertical_padding();
            this._resize_entry();
        });

    },

    set_text: function (text) {
        Meta.later_add(Meta.LaterType.BEFORE_REDRAW, () => {
            this.entry.set_text(text);
            this._resize_entry();
        });
    },

    _before_text_changed: function (added_text, length_of_added_text) {
        // If the text was pasted in (longer than 1 char) or is a newline, set
        // the sanitize flag to true so that after_text_changed cleans the line
        // breaks.
        if (length_of_added_text > 1 || /\r?\n/g.test(added_text))
            this.sanitize_flag = true;
    },

    _after_text_changed: function () {
        // remove line breaks
        if (this.sanitize_flag) {
            let txt = this.entry.get_text();
            this.entry.set_text(txt.replace(/[\r\n]/g, ' '));
            this.sanitize_flag = false;
        }

        this._resize_entry();
    },

    insert_newline: function () {
        if (this.single_line_mode) return Clutter.EVENT_PROPAGATE;

        let pos = this.entry.clutter_text.cursor_position;

        if (pos === -1) {
            this.entry.text += '\n';
        } else {
            this.entry.text = this.entry.text.slice(0, pos) + '\n' + this.entry.text.slice(pos);
            this.entry.clutter_text.set_selection(pos+1, pos+1);
        }
    },

    _resize_entry: function () {
        let theme_node     = this.entry.get_theme_node();
        let a              = this.entry.get_allocation_box();
        let [, nat_height] = this.entry.clutter_text.get_preferred_height(theme_node.adjust_for_width(a.x2 - a.x1));
        let h              = nat_height + theme_node.get_vertical_padding()*2;

        let entry_h = theme_node.adjust_for_height(a.y2 - a.y1);
        if (this.keep_min_height || entry_h <= h) this.entry.set_height(h);
    },
});
